%function [fundValueGrid, possibleGridMovements, gridMovementProb, expUtility, maxUtility2, maximizingKappa2, terminalWealth, terminalUtility, theta, lambda, Z, certEq, augTerminalWealth] = masterSim2(initialFundValue, fundShare, incFee, mgtFee, trp, certEq1, noStages)
function [possibleGridMovements, oneDayGridMovementProb, gridMovementProb, terminalUtility, maximizingKappa2] = masterSim2(initialFundValue, fundShare, incFee, mgtFee, trp, certEq, noStages, detailed)
    %INPUT:
    %trp = 2: trp,  0: prospect theory, -1: utility theory
    
    %mgtFee and incFee: quarterly rates
    
    %certEq: certaintyEquivalent vector to be augmented to terminalWealth
    %vector with multiple stages
    
    %noStages: number of stages to run for
    
    %detailed = 0: crude kappa set, 1: fine kappa set up to kappa = 250,
    %2:fine kappa set up to kappa = 60
    
    if(detailed ~= 1)
        kappaSet = [0; 1; 1.5; 2; 2.5; 3; 3.5; 4; 5; 6; 7; 8; 10; 20; 30; 40; 50; 60; 70; 80; 90];
        %kappaSet = [0; 1; 1.5; 2; 2.5; 3; 3.5; 4; 5; 6; 7; 8; 10; 20; 30; 40; 50; 60; 70; 80; 100; 120; 140; 160; 180; 200; 250];
    end
    
    if(detailed == 1) %maxkappa = 250
        kappaSet = zeros(500);
        kappaSet(1, 1) = 0;
        for i = 1:500
            kappaSet(i, 1) = (i+1)*0.5;
        end
    end
    
    if(detailed == 2) %maxkappa = 60
        kappaSet = zeros(500);
        kappaSet(1, 1) = 0;
        for i = 2:500
            kappaSet(i, 1) = (i-2)*0.125 + 1;
        end
    end
    
    noDays = 63;
    
    %START ASSET MOVEMENT PARAM
    %hodderJackwerth:
    mu = 2.8e-4; %risky asset daily mean return
    sigma = 0.0031; %risky asset daily standard dev
    riskFree = 2e-4; %risk-free asset daily return
    %kouwenbergZiemba:
%     mu = 2.38e-4;
%     sigma = 0.0126;
%     riskFree = 1.58e-4;

    %kouwenberg annual:
%     mu = 9.52e-4;
%     sigma=0.2/sqrt(63);
%     riskFree = 0.04/63;
    %END ASSET MOVEMENT PARAM

    benchmark = exp(riskFree*(noDays-1));
    
    %utility theory constants:
    gamma = 4;
    
    %prospect theory constants:
    gamma1 = 0.88;
    gamma2 = 0.88;
    A = 2.25;
    initialWealth = fundShare*initialFundValue;
    Z = ((1+riskFree)^(noDays-1))*(initialWealth-fundShare*initialFundValue); %wealth outside of fund at terminal time (assume it simply grows at the risk-free rate)
    theta = fundShare*benchmark + mgtFee*(1-fundShare)*benchmark + Z;
    
    %TRP constants:
    lambda = fundShare*0.6*benchmark + mgtFee*(1-fundShare)*0.6*benchmark + Z; %L = 0.6
%   lambda = fundShare*0.6*initialFundValue + mgtFee*(1-fundShare)*0.6*initialFundValue + Z; %L = 0.6
    tauLambda = -A*((theta)^gamma2)/lambda;
    
    %fund value grid constants:
    deltaLogX = log(10)/199; %setting the grid fund value delta rate
    fundGridSize = 301; %setting the fund value grid size
    fundGridMin = 0.1; %setting the lowest possible fund value for the grid
    
    disp('theta = ');
    disp(theta);
    disp('lambda = ');
    disp(lambda);
    disp('tauLambda = ');
    disp(tauLambda);
    
    function fundValueGrid = setFundValueGrid(noDays)
        fundValueGrid = zeros(fundGridSize, noDays); 
        
        fundValueGrid(1, 1) = fundGridMin;
        for i = 2:fundGridSize
            fundValueGrid(i, 1) = fundValueGrid(i-1, 1)*exp(deltaLogX);
        end
        
        for i = 2:noDays
            for j = 1:fundGridSize
                fundValueGrid(j, i) = fundValueGrid(j, 1)*exp((i-1)*riskFree);
            end
        end
    end
    fundValueGrid = setFundValueGrid(noDays);
    
    function terminalWealth = setTerminalWealth()
        %setting the terminal wealth for each possible terminal fund value
        
        terminalWealth = zeros(fundGridSize, 1);
        
        for i = 1:fundGridSize
            %liquidation boundary:
            if (fundValueGrid(i, noDays) < 0.6 && trp == -1)
                    terminalWealth(i, 1) = fundShare*fundValueGrid(i, noDays) + (1-fundShare)*mgtFee*fundValueGrid(i, noDays);
                    %terminalWealth(i, 1) = fundShare*fundValueGrid(i, noDays);
            else
                terminalWealth(i, 1) = fundShare*fundValueGrid(i, noDays) + (1-fundShare)*mgtFee*fundValueGrid(i, noDays) + (1-fundShare)*incFee*max(0, fundValueGrid(i, noDays) - benchmark);
            end
        end
    end
    terminalWealth = setTerminalWealth();
        
    function augTerminalWealth = augmentTerminalWealth(certEq)
        %for use with multiple stages
        %augments the terminalWealth vector with the certEq of continuation
        
        augTerminalWealth = zeros(fundGridSize, 1);
        
        for i = 1:fundGridSize
            for j = 1:size(kappaSet, 1)
                augTerminalWealth(i, j) = terminalWealth(i, 1) + certEq(i, 1);
            end
        end
    end

    if(noStages == 2)
        terminalWealth = augmentTerminalWealth(certEq);
    end
   
    function [terminalUtility, maxTermUtility] = setTerminalUtility()
        %setting the terminalUtility vector with CRRA-utility
        
        terminalUtility = zeros(fundGridSize, 1);
        
        for i = 1:fundGridSize
            if ((terminalWealth(i, 1)^(1-gamma)-1)/(1-gamma) == 1)
                terminalUtility(i, 1) = log(terminalWealth(i, 1));
            else
                terminalUtility(i, 1) = (terminalWealth(i, 1)^(1-gamma)-1)/(1-gamma);
            end
        end
        
        maxTermUtility = max(terminalUtility);
        disp(maxTermUtility);
    end

    if trp == -1
        [terminalUtility, maxTerminalUtility] = setTerminalUtility();
    end
    
    function [terminalUtility, maxTermUtility] = setTerminalProspectValue()
        %setting the terminalUtility vector with PT
        terminalUtility = zeros(fundGridSize, 1);
        
        for i = 1:fundGridSize
            terminalWealth(i, 1) = terminalWealth(i, 1) + Z;
            if(terminalWealth(i, 1) > theta) %if above benchmark
                terminalUtility(i, 1) = (terminalWealth(i, 1) - theta)^gamma2;
            elseif(terminalWealth(i, 1) <= theta)
                terminalUtility(i, 1) = -A*(theta - terminalWealth(i, 1))^gamma1;
            end
        end
        
        maxTermUtility = max(terminalUtility);
        disp(maxTermUtility);
    end

    if (trp == 0)
        [terminalUtility, maxTerminalUtility] = setTerminalProspectValue();
    end
    
    function [terminalUtility, maxTermUtility] = setNonProbTerminalTriValue()
        %setting the terminalUtility vector with non probabilistic TRP (the
        %final version of TRP used in the paper)
        
        terminalUtility = zeros(fundGridSize, 1);
        
        for i = 1:size(terminalWealth, 1)
            if(terminalWealth(i, 1) <lambda)
                terminalUtility(i, 1) = tauLambda*lambda;
            elseif (terminalWealth(i, 1) >= lambda && terminalWealth(i, 1) < theta)
                terminalUtility(i, 1) = -A*(theta-terminalWealth(i, 1))^gamma1;
            elseif (terminalWealth(i, 1) == theta)
                terminalUtility(i, 1) = 0;
            elseif(terminalWealth(i, 1) > theta)
                terminalUtility(i, 1) = ((terminalWealth(i, 1)-theta)^gamma2);
            end
        end
         
        maxTermUtility = max(terminalUtility);
        disp(maxTermUtility);
    end

    if trp == 2
        [terminalUtility, maxTerminalUtility] = setNonProbTerminalTriValue();
    end
    
    function possibleGridMovements = setPossibleGridMovements2()
        %calculating the possible grid movements for a given number of days
        possibleGridMovements = zeros(fundGridSize, noDays-1);
        
        for i = 1:fundGridSize
            for j = 1:(noDays-1)
                possibleGridMovements(i, j) = log(fundValueGrid(i, j + 1)/fundValueGrid(ceil(fundGridSize/2), 1));
            end
        end
    end
    possibleGridMovements = setPossibleGridMovements2();

    function [gridMovementProb, gridMovements] = setGridMovementProb(noDays)
        %setting the probability for the fund value to change with a chosen
        %set of rates over a deltaT from 1 to 90
        gridMovementProb = zeros(fundGridSize, size(kappaSet, 1), noDays-1);
        gridMovementProbSum = zeros(size(kappaSet, 1), 1, noDays-1);
        
        gridMovements = zeros(fundGridSize, noDays-1);
        for i = 2:size(kappaSet, 1) %if kappa = 0 is in the set start from 2
            for j = 1:fundGridSize
                for k = (noDays-1):-1:1
                    %calculating the "specific" mu for the chosen kappa and
                    %remaining time:
                    specMu = ((kappaSet(i, 1))*mu + (1-kappaSet(i, 1))*riskFree - 0.5*(kappaSet(i, 1))^2*sigma^2)*k;
                    %calculating the "specific" sigma for the chosen kappa and
                    %remaining time:
                    specSigma = kappaSet(i, 1)*sigma*sqrt(k);
                    
                    gridMovementProb(j, i, k) = (1/(sqrt(2*pi)*specSigma))*exp(-0.5*((possibleGridMovements(j, k) - specMu)/specSigma)^2);
                    gridMovementProbSum(i, 1, k) = gridMovementProbSum(i, 1, k) + gridMovementProb(j, i, k);
                end
            end
        end
        
        %dividing by the sum of probabilities for each kappa and deltaT:
        for i = 2:size(kappaSet, 1)%if kappa = 0 is in the set start from 2
            for j = 1:fundGridSize
                for k = (noDays-1):-1:1
                    gridMovementProb (j, i, k) = gridMovementProb(j, i, k)/gridMovementProbSum(i, 1, k);
                end
            end
        end
        
        %if kappa = 0 is in the set:
        gridMovementProb(ceil(fundGridSize/2), 1, :) = 1;
    end
    gridMovementProb = setGridMovementProb(noDays);
    
    function [expUtility, maximizingKappa2] = setExpectedUtilities2()
        expUtility = zeros(fundGridSize, noDays, size(kappaSet, 1));
        maxUtility2 = zeros(fundGridSize, noDays);
        maximizingKappa2 = zeros(fundGridSize, noDays);
        for i = (noDays-1):-1:1 %starting at the second last day
            %disp(i);
            for j = 1:fundGridSize %looping through current fund values
                    for k = 1:size(kappaSet, 1)
                        for l = 1:fundGridSize %looping through possible movements
                            terminalVal = fundValueGrid(j, i)*exp(possibleGridMovements(l, noDays-i));
                            
                            terminalIndex = j + l - ceil(fundGridSize/2);
                            if (terminalIndex <= fundGridSize && terminalIndex >= 1)
                                 if (trp == 0)
                                    %expUtility(j, i, k) = expUtility(j, i, k) + terminalUtility(terminalIndex, 1)*gridMovementProb(l, k, noDays-i);
                                    expUtility(j, i, k) = expUtility(j, i, k) + terminalUtility(terminalIndex, 1)*gridMovementProb(l, k, noDays-i);
                                 else
                                    expUtility(j, i, k) = expUtility(j, i, k) + terminalUtility(terminalIndex, 1)*gridMovementProb(l, k, noDays-i);
                                 end
                            end
                        end
                    end
                    
                %finding the maximum utility and maximizing kappa:
                [maxUtility2(j, i), index] = max(expUtility(j , i, :));
                maximizingKappa2(j, i) = kappaSet(index);
            end
        end
    end
    [expUtility, maximizingKappa2] = setExpectedUtilities2();
    
    oneDayGridMovementProb = gridMovementProb(:, :, 1);
end