function [maximizingKappaForPeriod, certEqMat, meanTerminalUtilMat] = multiPeriod(oneDayGridMovementProb, fundValueGrid)
    %runs the multi period analysis

    noPeriods = 16;
    
    fundShare = 0.1;
    trp = 2;
    
    tauMinus = -2.25;
    tauPlus = 1;
    deltaMinus = 0.92;
    deltaPlus = 1;
    benchmark = 1;
    mgtFee = 0.02*0.25;
    Z = 0;
    initialFundValue = 1;
    theta = fundShare*benchmark + mgtFee*(1-fundShare)*benchmark + Z;
    lambda = fundShare*0.60*benchmark + mgtFee*(1-fundShare)*0.60*benchmark + Z;
    L = tauMinus*(theta - lambda)^deltaMinus; %point of utility where liquidation boundary hits
    
    
    tauLambda = -2.25*((theta)^0.88)/lambda;
    
    maximizingKappaForPeriod = zeros(301, 63, noPeriods);
    certEqMat = zeros(301, noPeriods-1);
    meanTerminalUtilMat = zeros(301, noPeriods-1);
    
    [~, ~, ~, mgrUtility, ~] = masterSim2(1, fundShare, 0.2*0.25, 0.02*0.25, trp, 0, 1, 0);
    
    certEq = 0;
    
    for i = 1:noPeriods
        %computing maxKappa-surface:
        if(i == 1)
            [~, ~, ~, ~, maximizingKappaForPeriod(:, :, i)] = masterSim2(1, fundShare, 0.2*0.25, 0.02*0.25, trp, certEq, 1, 0);
        elseif(i==noPeriods)
            [~, ~, ~, ~, maximizingKappaForPeriod(:, :, i)] = masterSim2(1, fundShare, 0.2*0.25, 0.02*0.25, trp, certEq, 2, 2); %CHANGE DETAILED PARAM HERE IF NEEDED!
        else
            [~, ~, ~, ~, maximizingKappaForPeriod(:, :, i)] = masterSim2(1, fundShare, 0.2*0.25, 0.02*0.25, trp, certEq, 2, 0);
        end
            
        %breaking loop if this is last period in the recursive call:
        if(i == noPeriods)
            break;
        end
        
        %computing certaintyEquvalents:
        %'adding' certaintyEquivalents to terminal utility vector:
        
        for j = 156:200
            [meanTerminalUtilMat(j, i), ~, ~, ~] = utilDevMC(oneDayGridMovementProb, mgrUtility, maximizingKappaForPeriod(:, :, i), fundValueGrid, j);
        end
        meanTerminalUtilMat(1:155, i) = -L;
        meanTerminalUtilMat(201:301, i) = meanTerminalUtilMat(200, i);
        
        certEq = trpCertEqSolver(meanTerminalUtilMat(:, i));
        
        %certEq(1:155, 1) = 0;
        
        certEqMat(:, i) = certEq;
        
        [~, ~, ~, mgrUtility, ~] = masterSim2(1, fundShare, 0.2*0.25, 0.02*0.25, trp, certEq, 2, 0);
    end
end