function [terminalWealth, terminalValueOne, terminalValueTwo, theta, lambda, tauLambda] = plotValueFunc()
    %plots the value function 

    fundShare = 0.1;
    initialFundValue = 1;
    mgtFee = 0.02*0.25;
    incFee = 0.2*0.25;
    benchmark = 1;
    
    gamma1 = 0.88;
    gamma2 = 0.88;
    A = 2.25;
    Z = 0;
    theta = fundShare*benchmark + mgtFee*(1-fundShare)*benchmark + Z;
    lambda = fundShare*0.60*initialFundValue + mgtFee*(1-fundShare)*0.60*initialFundValue + Z; %assuming a 60 % liq bound
    
    %zero external compensation-based tauLambda:
    tauLambda = -A*((theta-0)^0.88)/lambda;
    %min compensation:
    %tauLambda = -A*((theta-fundShare*0.6)^0.88)/lambda;
    
    terminalValue = zeros(301, 1);
    
    function [terminalWealth] = createLinearWealthVec()
        fundValues = zeros(301, 1);
        terminalWealth = zeros(301, 1);

        %for illustrative purposes only:
        for i = 1:301
            terminalWealth(i, 1) = i*6.66666e-4;
%             terminalWealth(i, 1) = i *3.3333e-5;
        end
    end
    terminalWealth = createLinearWealthVec();

    function [terminalValue] = runTwoRef() %probabilistic version, not in use
        for i = 1:size(terminalWealth, 1)
            if (terminalWealth(i, 1) < theta)
                terminalValue(i, 1) = (1-normcdf(terminalWealth(i, 1), lambda, sigmaLambda))*-5*lambda + normcdf(terminalWealth(i, 1), lambda, sigmaLambda)*(-2.25*(theta-terminalWealth(i, 1))^0.92+terminalWealth(i, 1));
            elseif (terminalWealth(i, 1) == theta)
                terminalValue(i, 1) = terminalWealth(i, 1);
            elseif(terminalWealth(i, 1) > theta)
                terminalValue(i, 1) = ((terminalWealth(i, 1)-theta)^1+terminalWealth(i, 1));
            end
        end
        
        plot(terminalWealth, terminalValue);
    end

    function [terminalValue] = runTwoRefLin() %piecewise linear version
        for i = 1:size(terminalWealth, 1)
            if(terminalWealth(i, 1) <lambda)
                terminalValue(i, 1) = tauLambda*lambda;
            elseif (terminalWealth(i, 1) >= lambda && terminalWealth(i, 1) < theta)
                terminalValue(i, 1) = -2.25*(theta-terminalWealth(i, 1));
            elseif (terminalWealth(i, 1) == theta)
                terminalValue(i, 1) = terminalWealth(i, 1);
            elseif(terminalWealth(i, 1) > theta)
                terminalValue(i, 1) = 1*(terminalWealth(i, 1)-theta);
            end
        end 
        plot(terminalWealth, terminalValue);
    end

    function [terminalUtility] = runTwoRefNonProb() %non probabilistic version (final version used in the paper)
        for i = 1:size(terminalWealth, 1)
            if(terminalWealth(i, 1) <lambda)
                terminalUtility(i, 1) = tauLambda*lambda;
            elseif (terminalWealth(i, 1) >= lambda && terminalWealth(i, 1) < theta)
                terminalUtility(i, 1) = -A*(theta-terminalWealth(i, 1))^gamma1;
            elseif (terminalWealth(i, 1) == theta)
                terminalUtility(i, 1) = 0;
            elseif(terminalWealth(i, 1) > theta)
                terminalUtility(i, 1) = ((terminalWealth(i, 1)-theta)^gamma2);
            end
        end
        
        plot(terminalWealth, terminalUtility);
    end

    function [terminalValue] = runOneRef() %standard PT value func
        for i = 1:size(terminalWealth, 1)
            if (terminalWealth(i, 1) > theta)
                terminalValue(i, 1) = (terminalWealth(i, 1) - theta)^0.88;
            else
                terminalValue(i, 1) = -A*(theta - terminalWealth(i, 1))^0.88;
            end
        end
        plot(terminalWealth, terminalValue);
    end
    terminalValue = runOneRef();
    terminalValueOne = runOneRef();
    terminalValueTwo =runTwoRefNonProb();
    
    plot(terminalWealth, [terminalValueOne, terminalValueTwo]);
end