function [meanTerminalUtil, terminalUtil, terminalFundVal, gridCount, oneDayGridMovementCDF] = utilDevMC(oneDayGridMovementProb, investorTerminalUtility, maximizingKappa, fundValueGrid, initialFundIndex, certEq)
    %INPUT:
    %oneDayGridMovementProb = grid movement probabilities for a single day
    %ahead
    
    %investorTerminalUtility = the investor's PT terminal utility
    %vector
    
    %maximizingKappa = optimal kappa matrix for mgr or investor
    %fundValueGrid = fund value grid matrix (301x63)
    %initialFundIndex = the starting fund value index. standard choice = 200
    %certeq = not in use
    
    %OUTPUT:
    %meanTerminalUtil = mean terminal utility found with simulation
    %terminalUtil = vector of all sample terminal utilities
    %terminalFundVal = vector of all sample terminal fund values
    %gridCount = counts the number of times a certain grid point is visited
    
    %computing the_cumulative_oneDayGridMovementProbabilities:
    oneDayGridMovementCDF = zeros(size(oneDayGridMovementProb, 1), size(oneDayGridMovementProb, 2));
    for i = 1:size(oneDayGridMovementProb,2)
        for j = 1:size(oneDayGridMovementProb, 1)
            oneDayGridMovementCDF(j, i) = sum(oneDayGridMovementProb(1:j, i));
        end
    end
    
    maximizingKappaIndex = zeros(size(maximizingKappa, 1), size(maximizingKappa, 2));
    %assigning the correct maxKappa index to corresponding kappa-values
    %depends on the chosen kappaSet used in input
    for i = 1:size(maximizingKappaIndex, 1)
        for j = 1:size(maximizingKappaIndex, 2)
            switch maximizingKappa(i, j)
                case 0
                    maximizingKappaIndex(i, j) = 1;
                case 1
                    maximizingKappaIndex(i, j) = 2; 
                case 1.5
                    maximizingKappaIndex(i, j) = 3;
                case 2
                    maximizingKappaIndex(i, j) = 4;
                case 2.5
                    maximizingKappaIndex(i, j) = 5;
                case 3
                    maximizingKappaIndex(i, j) = 6;
                case 3.5
                    maximizingKappaIndex(i, j) = 7;
                case 4
                    maximizingKappaIndex(i, j) = 8;
                case 5
                    maximizingKappaIndex(i, j) = 9;
                case 6
                    maximizingKappaIndex(i, j) = 10;
                case 7
                    maximizingKappaIndex(i, j) = 11;
                case 8
                    maximizingKappaIndex(i, j) = 12;
                case 10
                    maximizingKappaIndex(i, j) = 13;
                case 20
                    maximizingKappaIndex(i, j) = 14;
                case 30
                    maximizingKappaIndex(i, j) = 15;
                case 40
                    maximizingKappaIndex(i, j) = 16;
                case 50
                    maximizingKappaIndex(i, j) = 17;
                case 60
                    maximizingKappaIndex(i, j) = 18;
                case 70
                    maximizingKappaIndex(i, j) = 19;
                case 80
                    maximizingKappaIndex(i, j) = 20;
                case 90
                    maximizingKappaIndex(i, j) = 21;
            end
        end
    end
    
    disp(maximizingKappaIndex);
    
    sampleSize = 1000000; %number of samples to run
    currentFundIndex = initialFundIndex; %the current fund value grid index (from 1 to 301)
    terminalUtil = zeros(sampleSize, 1);
    terminalFundVal = zeros(sampleSize, 1);
    gridCount = zeros(301, 62);
    
    for i = 1:sampleSize
        %prints progress every 100 000 samples:
        if(mod(i, 100000) == 0)
            disp(i);
        end
        
        currentFundIndex = initialFundIndex;
        for j = 1:62 %j = current day + 1
            kappaChoiceIndex = maximizingKappaIndex(currentFundIndex, j);
            gridCount(currentFundIndex, j) = gridCount(currentFundIndex, j) + 1;
            
            randomNo = rand(); %decides where the fund value moves next (grid movement)
            
            for k = 1:301
                if(randomNo < oneDayGridMovementCDF(k, kappaChoiceIndex))
                    currentFundIndex = currentFundIndex + (k - 151);
                    if(currentFundIndex > 301)
                        currentFundIndex = 301;
                    elseif(currentFundIndex < 1)
                        currentFundIndex = 1;
                    end
                    break;
                end
            end
        end
        terminalFundVal(i, 1) = fundValueGrid(currentFundIndex, 63);
        terminalUtil(i, 1) = investorTerminalUtility(currentFundIndex, 1);
    end
    meanTerminalUtil = mean(terminalUtil);
    
    
end